/*************************************************************************
 * GBA hardware tile map display information
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 ************************************************************************/

#ifndef	_GBA_BKG_H
#define	_GBA_BKG_H

#include "types.h"


// top of tile map memory (from here to 0x06010000 is used for both tiles and tile maps for backgrounds)
#define	VRAM_BKG_TOP				0x06000000

// tile map settings registers
#define pREG_BKG_0_SET				(u16 *)0x4000008	// register for setting tilemap 0
#define REG_BKG_0_SET				*pREG_BKG_0_SET

#define pREG_BKG_1_SET				(u16 *)0x400000A	// register for setting tilemap 1
#define REG_BKG_1_SET				*pREG_BKG_1_SET

#define pREG_BKG_2_SET				(u16 *)0x400000C	// register for setting tilemap 2
#define REG_BKG_2_SET				*pREG_BKG_2_SET

#define pREG_BKG_3_SET				(u16 *)0x400000E	// register for setting tilemap 3
#define REG_BKG_3_SET				*pREG_BKG_3_SET

// tile map settings (set in one of the REG_BKG_?_SET registers)
#define	BKG_PRIORITY_MASK			0x0003	// priority control
#define	BKG_BASE_BLOCK_MASK			0x000C	// select the character block to use (where in VRAM are the tiles?)
#define	BKG_BASE_BLOCK_SHIFT		2
#define	BKG_MOSAIC					0x0040	// enable mosaic on this tile map
#define	BKG_COLOR_MODE				0x0080	// enable 256 color palette (default is 16 color palette)
#define	BKG_SCREEN_BASE_BLOCK_MASK	0x1F00	// select the screen base block (where in VRAM is the map?)
#define BKG_SCREEN_BASE_BLOCK_SHIFT	8
#define	BKG_WRAP					0x2000	// enable screen wrap at edges of map (works on rotating maps only)
#define	BKG_MAP_SIZE_MASK			0xC000	// tile map size setting
#define	BKG_MAP_SIZE_SHIFT			14

#define	BKG_MAP_SIZE_0				0x0000	// 256x256, or 128x128 if rotating background
#define	BKG_MAP_SIZE_1				0x8000	// 512x256, or 256x256 if rotating background
#define	BKG_MAP_SIZE_2				0x4000	// 256x512, or 512x512 if rotating background
#define	BKG_MAP_SIZE_3				0xC000	// 512x512, or 1024x1024 if rotating background

// base block calculations... the following macro is used to define where in
// VRAM a set of tiles resides (each tile map can have a seperate block of tiles)
#define	VRAM_BASE_BLOCK_SIZE			0x4000
#define	VRAM_TILE_ADDRESS(blockNum)		(u32 *)(VRAM_BKG_TOP + (blockNum * VRAM_BASE_BLOCK_SIZE))

// screen base block calculations... the following macro is used to define
// where in VRAM the data for a tile map resides
#define	VRAM_SCREEN_BASE_BLOCK_SIZE		0x800
#define	VRAM_TILEMAP_ADDRESS(blockNum)	(u16 *)(VRAM_BKG_TOP + (blockNum * VRAM_SCREEN_BASE_BLOCK_SIZE))

// map scroll registers
#define pREG_BKG_0_SCROLL_X			(u16 *)0x04000010	// register for scrolling map 0 horizontally
#define REG_BKG_0_SCROLL_X			*pREG_BKG_0_SCROLL_X
#define pREG_BKG_0_SCROLL_Y			(u16 *)0x04000012	// register for scrolling map 0 vertically
#define REG_BKG_0_SCROLL_Y			*pREG_BKG_0_SCROLL_Y

#define pREG_BKG_1_SCROLL_X			(u16 *)0x04000014	// register for scrolling map 1 horizontally
#define REG_BKG_1_SCROLL_X			*pREG_BKG_1_SCROLL_X
#define pREG_BKG_1_SCROLL_Y			(u16 *)0x04000016	// register for scrolling map 1 vertically
#define REG_BKG_1_SCROLL_Y			*pREG_BKG_1_SCROLL_Y

#define pREG_BKG_2_SCROLL_X			(u16 *)0x04000018	// register for scrolling map 2 horizontally
#define REG_BKG_2_SCROLL_X			*pREG_BKG_2_SCROLL_X
#define pREG_BKG_2_SCROLL_Y			(u16 *)0x0400001A	// register for scrolling map 2 vertically
#define REG_BKG_2_SCROLL_Y			*pREG_BKG_2_SCROLL_Y

#define pREG_BKG_3_SCROLL_X			(u16 *)0x0400001C	// register for scrolling map 3 horizontally
#define REG_BKG_3_SCROLL_X			*pREG_BKG_3_SCROLL_X
#define pREG_BKG_3_SCROLL_Y			(u16 *)0x0400001E	// register for scrolling map 3 vertically
#define REG_BKG_3_SCROLL_Y			*pREG_BKG_3_SCROLL_Y

// tile reference settings
#define	TILE_IDX_MASK					0x03FF	// tile index for this tile in map
#define	TILE_FLIP_X						0x0400	// flip tile horizontally
#define TILE_FLIP_Y						0x0800	// flip tile vertically
#define	TILE_PALETTE_MASK				0xF000	// palette to use for this tile

#endif

