/***************************************************************************
 * GBA hardware sprite information
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 **************************************************************************/

#ifndef _GBA_SPRITE_H
#define _GBA_SPRITE_H

#include "types.h"

// location of OAM Table in VRAM
#define VRAM_OAM_TABLE				0x7000000

// top of sprite tiles memory in VRAM
#define	VRAM_SPRITE_TILES			0x6010000

// sprite attribute 0 bit settings
#define	SPRITE_Y_MASK				0x00FF	// screen Y coordinate

#define SPRITE_ROTATION_SCALE		0x0100	// rotation scaling flag
#define SPRITE_SIZE_DOUBLE			0x0200	// size double flag

#define	SPRITE_ALPAHBLEND_MODE_MASK			0x0C00	// alpha blend settings
#define SPRITE_ALPAHBLEND_MODE_NORMAL     	0x0000
#define SPRITE_ALPAHBLEND_MODE_TRANSPERANT	0x0400
#define SPRITE_ALPAHBLEND_MODE_WINDOWED		0x0800

#define SPRITE_MOSAIC				0x1000	// mosaic enable flag

#define SPRITE_COLOR_MODE_16		0x0000	// 16 color palette
#define SPRITE_COLOR_MODE_256		0x2000	// 256 color palette

#define	SPRITE_SHAPE_MASK			0xC000	// sprite shape settings
#define SPRITE_SHAPE_SQUARE			0x0000
#define SPRITE_SHAPE_TALL			0x4000
#define SPRITE_SHAPE_WIDE			0x8000

// sprite attribute 1 bit settings
#define	SPRITE_X_MASK				0x01FF	// screen X coordinate

#define	SPRITE_ROTDATA_REF_MASK		0x0E00	// if rotation enabled, then this is the rotation settings index
#define	SPRITE_ROTDATA_SHIFT		9
#define	SPRITE_HFLIP				0x1000	// if no rotation, this bit is the horizontal flip flag
#define	SPRITE_VFLIP				0x2000	// if no rotation, this bit is the vertical flip flag

#define	SPRITE_SIZE_MASK			0xC000	// sprite size mask
#define SPRITE_SIZE_8				0x0000
#define SPRITE_SIZE_16				0x4000
#define SPRITE_SIZE_32				0x8000
#define SPRITE_SIZE_64				0xC000

// sprite attribute 2 bit settings
#define	SPRITE_TILE_REF_MASK		0x03FF	// tile reference

#define	SPRITE_PRIORITY_MASK		0x0C00	// sprite priority
#define	SPRITE_PRIORITY_SHIFT		10

#define	SPRITE_PALETTE_MASK			0xF000	// palette number
#define	SPRITE_PALETTE_SHIFT		12


#define	NUM_OAM_SPRITES		128		// number of hardware sprites the GBA can handle

// OAM hardware sprite
typedef struct SpriteOAMStruct
{
	u16 attribute0;
	u16 attribute1;
	u16 attribute2;
	u16 attribute3;

} SpriteOAMStruct;

typedef struct SpriteOAMRotateStruct
{
	u16 unused1[3];
	u16 xcos;

	u16 unused2[3];
	u16 ysin;

	u16 unused3[3];
	u16 xnsin;

	u16 unused4[3];
	u16 ycos;
} SpriteOAMRotateStruct;


#endif

