/***********************************************************************
 * interrupt handler code
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 **********************************************************************/


#include "interrupt.h"
#include "vblank.h"



// jump table for the interrupt functions (name must be IntrTable for gcc boot code to find it)
InterruptFunction IntrTable[]  =
{
	Interrupt_VBlank,	// vblank
	NULL,				// hblank
	NULL,				// vcount
	NULL,				// timer 0
	NULL,				// timer 1
	NULL,				// timer 2
	NULL,				// timer 3
	NULL,				// serial communications port
	NULL,				// DMA 0
	NULL,				// DMA 1
	NULL,				// DMA 2
	NULL,				// DMA 3
	NULL,				// keyboard
	NULL,				// cartridge
};


void Interrupt_VBlank (void);


/************************************************************
 * enable interrupts
 *
 * IN:		bits for what interrupts to enable:
 * 				INTERRUPT_VBLANK
 * 				INTERRUPT_HBLANK
 *				INTERRUPT_VCOUNT
 *				INTERRUPT_TIMER0
 *				INTERRUPT_TIMER1
 *				INTERRUPT_TIMER2
 *				INTERRUPT_TIMER3
 *				INTERRUPT_SERIAL
 *				INTERRUPT_DMA0
 *				INTERRUPT_DMA1
 *				INTERRUPT_DMA2
 *				INTERRUPT_DMA3
 *				INTERRUPT_KEYBOARD
 *				INTERRUPT_CART
 ************************************************************/
void Interrupt_Enable (u16 settings)
{
	if (settings == 0)
	{	// no interrupts passed in... don't enable anything
		return;
	}

	if (settings & INTERRUPT_VBLANK)
	{	// vblank interrupt also requires an enable in the REG_INTERRUPT_STATUS register
		REG_INTERRUPT_STATUS |= INTERRUPT_STATUS_VBLANK_ENABLE;
	}

	if (settings & INTERRUPT_HBLANK)
	{	// hblank interrupt also requires an enable in the REG_INTERRUPT_STATUS register
		REG_INTERRUPT_STATUS |= INTERRUPT_STATUS_HBLANK_ENABLE;
	}

	if (settings & INTERRUPT_VCOUNT)
	{	// vcount interrupt also requires an enable in the REG_INTERRUPT_STATUS register
		REG_INTERRUPT_STATUS |= INTERRUPT_STATUS_VCOUNT_ENABLE;
	}

	// enable the specific interrupts
	REG_INTERRUPT_ENABLE |= settings;

	// enable interrupts in general in the master enable register
	REG_INTERRUPT_MASTER_ENABLE |= INTERRUPT_MASTER_ENABLE;
}


/*************************************************************
 * disable interrupts
 *
 * IN:		bits for what interrupts to disable
 * 				0 = disable all interrupts (including master switch)
 *************************************************************/
void Interrupt_Disable (u16 settings)
{
	// disable the specific interrupts
	REG_INTERRUPT_ENABLE &= ~settings;

	if (settings == 0)
	{	// all interrupts disabled... clear the master enable switch
		REG_INTERRUPT_MASTER_ENABLE &= ~INTERRUPT_MASTER_ENABLE;
	}
}


