/***************************************************************
 * math tables and funtions headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 **************************************************************/

#ifndef	_MATH_H
#define	_MATH_H

#include "types.h"

// return the absolute value of a number (any signed format)
#define	ABS(num)             		((num >= 0) ? num : -num)

// return the minimum of two integers
#define	MIN(num1, num2)				((num1 <= num2) ? num1 : num2)

// return the maximum of two integers
#define	MAX(num1, num2)				((num1 >= num2) ? num1 : num2)


// routines from the mathAsm.s file:

/***************************************************
 * integer divide
 **************************************************/
s32 s32_div (s32 numerator, s32 denominator);

/***************************************************
 * integer modulus
 **************************************************/
s32 s32_umod (s32 numerator, s32 denominator);




// fixed point values, 24 bits integer, 8 bit fractional
typedef s32 s24f8;

// 2D vector notation
typedef struct s24f8_vect2D
{
	s24f8	x, y;

} s24f8_vect2D;


#define	s24f8_ONE					0x00000100
#define s24f8_POS_INFINITY			0x7FFFFFFF
#define s24f8_NEG_INFINITY			0x80000000


/*********************************************************
 * convert to/from s24f8 fixed point format and other
 * formats (int, float)
 ********************************************************/
#define	s24f8_to_int(num)			((num)>>8)
#define	int_to_s24f8(num)			((num<<8))
#define	float_to_s24f8(num)			((s24f8)((num) * 256.0f))


/*********************************************************
 * multiply two fixed point numbers
 *
 * IN:		num1, num2 - s24f8 numbers
 *
 * RETURNS:	num1 * num2 (in s24f8 format)
 *
 * NOTES:	does not check for overflow!
 ********************************************************/
#define	s24f8_mul(numF,numI)		(((numF) * (numI))>>8)


/*********************************************************
 * multiply a fixed point number with an integer
 *
 * IN:		numF = s24f8 number
 * 			numI = int number
 *
 * RETURNS:	numF * numI (in s24f8 format)
 *
 * NOTES:	does not check for overflow!
 ********************************************************/
#define	s24f8_mulInt(num1,num2)	((num1) * (num2))


/*********************************************************
 * divide two fixed point numbers
 *
 * IN:		num, den - s24f8 numbers
 *
 * RETURNS:	num / den (in s24f8 format)
 *
 * NOTES:	does not check for overflow!
 ********************************************************/
#define	s24f8_div(num,den)		(s32_div(((num)<<8), den))


/******************************************************************
 * get an approximate distance between two 2D points, using quicky
 * code
 *
 * IN:		the two points
 *
 * RETURNS:	approx. distance between the two points
 ******************************************************************/
extern s24f8 s24f8vect2D_approxDist (s24f8_vect2D *pA, s24f8_vect2D *pB);


/********************************************
 * get the angle between two points
 *
 * IN:		pA, pB = 2 points
 *
 * RETURNS:	angle
 *******************************************/
extern ANGLE Math_GetAngleFromPoints (s24f8_vect2D *pA, s24f8_vect2D *pB);


// sin and cosine tables:
extern s24f8 SIN_TABLE[256];
extern s24f8 COS_TABLE[256];


// quicky random number generator
extern void Random_ReSeed (u32 seed);
extern u32 Random_Get32 (void);


#endif

